;+
;procedure: gse2sse
;
;Purpose: Coordinate transformation between GSE & SSE coordinates(and the inverse)
;
;     SSE is defined as:
;        X: Moon->Sun Line projected into the ecliptic plane
;        Y: Z x X
;        Z: Ecliptic north
;         
;     GSE is defined as:
;        X: Earth Sun Line(naturally in the ecliptic plane)
;        Y: Z x X
;        Z: Ecliptic North
;inputs:
;
;  name_in: 
;    Name of input tplot variable to be transformed
;  name_sun_pos:
;    Name of the solar position tplot variable in GEI coordinates
;  name_lun_pos:
;    Name of the lunar position tplot variable in GEI coordinates
;  name_out:
;    Name that the rotated variable should take.
;    
;keywords:
;
;   /SSE2GSE inverse transformation
;
;   /IGNORE_DLIMITS: Dlimits normally used to determine if coordinate
;   system is correct, to decide if position needs offset, or to 
;   stop incorrect transforms.  This option will stop this behavior. 
;
;Examples:
;      gse2sse,'tha_state_pos','slp_sun_pos_gse','slp_lun_pos_gse','tha_state_pos_sse'
;      gse2sse,'tha_state_pos_sse','slp_sun_pos_gse','slp_lun_pos_gse','tha_state_pos_gse',/sse2gse,/ignore_dlimits
;
;Notes:
;   #1 If dlimit-labeled position passed in without /ignore_dlimits,
;      input will be offset to account for relative position of frames of reference.
;   #2 If dlimit-labeled velocity passed in without /ignore_dlimits,
;      input will be offset to account for relative motion of frames of reference
;   #3 If dlimit-labeled acceleration passed in without /ignore_dlimits,
;      warning will be raise, but offset will not be applied automatically
;   #4 Uses tvector_rotate, and sse_matrix_make to perform the rotation.
;      tvector_rotate will also interpolate the rotation matrix onto the time-grid of the input.
;      Interpolation done using quaterions and the spherical linear interpolation algorithm (SLERP)
;      
;Written by Jenni Kissinger and Patrick Cruce
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2009-08-14 16:38:38 -0700 (Fri, 14 Aug 2009) $
; $LastChangedRevision: 6586 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/cotrans/gse2sse.pro $
;-



pro gse2sse,name_in,name_sun_pos,name_lun_pos,name_out,sse2gse=sse2gse,ignore_dlimits=ignore_dlimits

  compile_opt idl2
  
  tn1 = tnames(name_sun_pos)
  tn2 = tnames(name_lun_pos)
  
  if n_elements(tn1) ne 1 || n_elements(tn2) ne 1 || tn1[0] eq '' || tn2[0] eq '' then begin
    message,'Aborted: Must load Sun/Moon position to perform this transformation'
  endif
  
  cotrans,name_sun_pos,name_sun_pos+'_gse_cotrans',/gei2gse,ignore_dlimits=ignore_dlimits
  cotrans,name_lun_pos,name_lun_pos+'_gse_cotrans',/gei2gse,ignore_dlimits=ignore_dlimits

  sse_matrix_make,name_sun_pos+'_gse_cotrans',name_lun_pos+'_gse_cotrans',newname='sse_mat_cotrans',fail=fail,ignore_dlimits=ignore_dlimits
  
  if fail then begin
    message,'Failed to create SSE rotation matrix
  endif

  st_type = 'none'
  
  get_data,name_in,dlimit=dl
  str_element,dl,'data_att.st_type',st_type

  name_one = name_in
  name_two = name_out

  ;perform affine offsets for forward transform
  if ~keyword_set(sse2gse) then begin
    if st_type eq 'pos' then begin
      if keyword_set(ignore_dlimits) then begin
        dprint,'WARNING: dlimits indicates data is a position.  Full transformation between SSE coords requires offset that will not be performed because IGNORE_DLIMITS is set'
      endif else begin
        tinterpol_mxn,name_lun_pos+'_gse_cotrans',name_one,/overwrite,error=err
        if err eq 0 then begin
          message,'Error performing position offset during SSE transformation(interpolation operation)'
        endif
        
        calc,'"'+name_two+'" = "' + name_one + '"-"' + name_lun_pos+'_gse_cotrans"',error=err
        if keyword_set(err) then begin
          message,'Error performing position offset during SSE transformation(subtraction operation)'
        endif
        
        name_one = name_two
      endelse
    endif else if st_type eq 'vel' then begin
      if keyword_set(ignore_dlimits) then begin
        dprint,'WARNING: dlimits indicates data is a velocity.  Full transformation between SSE coords requires offset that will not be performed because IGNORE_DLIMITS is set'
      endif else begin
        
        deriv_data,name_lun_pos+'_gse_cotrans',newname=name_lun_pos+'_gse_deriv_cotrans'
        
        tinterpol_mxn,name_lun_pos+'_gse_deriv_cotrans',name_one,/overwrite,error=err
        if err eq 0 then begin
          message,'Error performing velocity offset during SSE transformation(interpolation operation)'
        endif
        
        calc,'"'+name_two+'" = "' + name_one + '"-"' + name_lun_pos+'_gse_deriv_cotrans"',error=err
        if keyword_set(err) then begin
          message,'Error performing velocity offset during SSE transformation(subtraction operation)'
        endif
        
        name_one = name_two
        
      endelse
    endif else if st_type eq 'acc' then begin
      dprint,'WARNING dlimits indicates data is an acceleration.  Full transformation between SSE coords requires offset that must be performed manually.'
    endif
  endif
  
  tvector_rotate,'sse_mat_cotrans',name_one,newname=name_two,error=err,invert=keyword_set(sse2gse)
  
  if err eq 0 then begin
  
    message,'Error performing rotation during SSE transformation
  
  endif
  
  ;performing affine offsets for inverse transform
  if keyword_set(sse2gse) then begin
    if st_type eq 'pos' then begin
      if keyword_set(ignore_dlimits) then begin
        dprint,'WARNING: dlimits indicates data is a position.  Full transformation between SSE coords requires offset that will not be performed because IGNORE_DLIMITS is set'
      endif else begin
        tinterpol_mxn,name_lun_pos+'_gse_cotrans',name_one,/overwrite,error=err
        if err eq 0 then begin
          message,'Error performing position offset during SSE transformation(interpolation operation)'
        endif
        
        calc,'"'+name_two+'" = "' + name_one + '"+"'+name_lun_pos+'_gse_cotrans"',error=err
        if keyword_set(err) then begin
          message,'Error performing position offset during SSE transformation(subtraction operation)'
        endif
        
        name_one = name_two
      endelse
    endif else if st_type eq 'vel' then begin
      if keyword_set(ignore_dlimits) then begin
        dprint,'WARNING: dlimits indicates data is a velocity.  Full transformation between SSE coords requires offset that will not be performed because IGNORE_DLIMITS is set'
      endif else begin
        
        deriv_data,name_lun_pos+'_gse_cotrans',newname=name_lun_pos+'_gse_deriv_cotrans'
        
        tinterpol_mxn,name_lun_pos+'_gse_deriv_cotrans',name_one,/overwrite,error=err
        
        if err eq 0 then begin
          message,'Error performing velocity offset during SSE transformation(interpolation operation)'
        endif
        
        calc,'"'+name_two+'" = "' + name_one + '"+"'+name_lun_pos+'_gse_deriv_cotrans"',error=err
        if keyword_set(err) then begin
          message,'Error performing velocity offset during SSE transformation(subtraction operation)'
        endif
        
        name_one = name_two
        
      endelse
    endif else if st_type eq 'acc' then begin
      dprint,'WARNING dlimits indicates data is an acceleration.  Full transformation between SSE coords requires offset that must be performed manually.'
    endif
  endif
  
  
end

